(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: relation.mli,v 1.39 2008/02/12 22:13:02 blanqui Exp $ *)

(** {3 Operations on relations of relational types.} *)

open Parsetree;;

(** {6 Test functions on relations.} *)

val is_rewrite : relation -> bool;;
val is_commutative : relation -> bool;;
val is_associative : relation -> bool;;
(*val is_absorbent : relation -> bool;;*)
val is_absorbing : relation -> bool;;
val is_neutral : relation -> bool;;
val is_idempotent : relation -> bool;;
val is_inverse : relation -> bool;;
val is_distributive : relation -> bool;;
val is_nilpotent : relation -> bool;;

(** {6 Test functions on lists of relations.} *)

val has_commutative : relations -> bool;;
val has_associative : relations -> bool;;
val has_idempotent : relations -> bool;;
val has_absorbing : relations -> bool;;
val has_neutral : relations -> bool;;
val has_nilpotent : relations -> bool;;

(** {6 Selection functions on lists of relations.} *)

val rewrite : relations -> (pattern * expression) list;;
val commutative : relations -> Longident.t option;;
val associative : relations -> relation_side option;;
(*val absorbent : relations -> (relation_side * generator) option;;*)
val absorbing : relations -> (relation_side * generator) option;;
val neutral : relations -> (relation_side * generator) option;;
val idempotent : relations -> relation_side option;;
val inverse : relations ->
  (relation_side * generator * generator option) option;;
val distributive : relations ->
  (relation_side * generator * generator option * distributivity) list;;
val nilpotent : relations -> generator option;;

(** {6 Test functions on generators and lists of relations.} *)

val is_neutral_gen : generator -> relations -> bool;;

(** {6 Comparison functions.} *)

val precedence : relation -> int;;

val compare_generator : generator -> generator -> int;;
(** Compare generators. *)

val compare_relation : (relation -> int) -> relation -> relation -> int;;
(** Comparaison function on relations based on precedence returning 0
when comparing two rewrite's, two structure_item's, two commutative's
or two associative's. *)
