(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_unary.mli,v 1.9 2008/02/11 23:05:59 weis Exp $ *)

(** {3 Generation of construction function for unary generators.} *)

val genr_function :
  Genr_base.function_definition ->
  (Code.lident * Code.pattern list * Code.exp) list

(** We treat the case of unary generators.

    Let's [g] be such a generator; we generate a unary function that
    normalizes terms that are built with [g] as their head constructor.

    We perform here the generation of code for the construction function of a
    ``true'' unary generator.

    A true unary generator has one argument and this argument is
    not of type [list]. Hence, this argument should not be treated as the
    variable length list of arguments of an associative binary operation.

    The case of an associative binary operator is handled in the module
    [Genr_vary]; we branch to this case when encountering an associative
    operator, since a variable length list of argument cannot exist for a
    true unary operator. *)
;;
