(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* term data structure *)

open Var;;
open Symb;;

type term =
  | Var of var
  | App of symb * term list
;;

let rec vars = function
  | Var x -> VarSet.singleton x
  | App (_, ts) -> vars_of_terms ts

and vars_of_terms = function
  | [] -> VarSet.empty
  | t :: ts -> VarSet.union (vars t) (vars_of_terms ts)
;;

let is_linear =
  let vset = ref VarSet.empty in
  let rec aux = function
    | Var x ->
	begin
	  match VarSet.mem x !vset with
	    | true -> false
	    | false -> vset := VarSet.add x !vset; true
	end
    | App (_, ts) -> List.for_all aux ts
  in
    fun t ->
      let b = aux t in vset := VarSet.empty; b
;;

let rec symbols = function
  | Var _ -> SymbSet.empty
  | App (f, ts) -> SymbSet.add f (symbols_of_terms ts)

and symbols_of_terms = function
  | [] -> SymbSet.empty
  | t :: ts -> SymbSet.union (symbols t) (symbols_of_terms ts)
;;

let occurs x =
  let rec aux = function
    | Var y -> y = x
    | App (_, ts) -> List.exists aux ts
  in aux
;;

open Printf;;

let rec bprint b = function
  | Var x -> bprintf b "%s" (string_of_var x)
  | App (f, []) -> bprintf b "%s" (string_of_symbol f)
  | App (f, ts) -> bprintf b "(%s%a)" (string_of_symbol f) bprint_terms ts

and bprint_terms b = List.iter (bprintf b " %a" bprint)
;;
